import contextlib
import logging
import os
from pathlib import Path

import dotenv
from navigator import Navigator

logger = logging.getLogger()


class RunNotebook:
    def __init__(self, navigator: Navigator):
        self.nav = navigator
        self.nav.initialize

    def run_notebook(
        self, path, expected_output_text, conda_env, runtime=30000, retry=2
    ):
        """Run jupyter notebook and check for expected output text anywhere on
        the page.

        Note: This will look for and exact match of expected_output_text
        _anywhere_ on the page so be sure that your text is unique.

        Conda environments may still be being built shortly after deployment.

        conda_env: str
            Name of conda environment. Python conda environments have the
            structure "conda-env-nebari-git-nebari-git-dashboard-py" where
            the actual name of the environment is "dashboard".
        """
        logger.debug(f">>> Running notebook: {path}")
        filename = Path(path).name

        # navigate to specific notebook
        file_locator = self.nav.page.get_by_text("File", exact=True)

        file_locator.wait_for(
            timeout=self.nav.wait_for_server_spinup,
            state="attached",
        )
        file_locator.click()
        self.nav.page.get_by_role("menuitem", name="Open from Path…").get_by_text(
            "Open from Path…"
        ).click()
        self.nav.page.get_by_placeholder("/path/relative/to/jlab/root").fill(path)
        self.nav.page.get_by_role("button", name="Open", exact=True).click()
        # give the page a second to open, otherwise the options in the kernel
        # menu will be disabled.
        self.nav.page.wait_for_load_state("networkidle")
        if self.nav.page.get_by_text(
            "Could not find path:",
            exact=False,
        ).is_visible():
            logger.debug("Path to notebook is invalid")
            raise RuntimeError("Path to notebook is invalid")
        # make sure the focus is on the dashboard tab we want to run
        self.nav.page.get_by_role("tab", name=filename).get_by_text(filename).click()
        self.nav.set_environment(kernel=conda_env)

        # make sure that this notebook is one currently selected
        self.nav.page.get_by_role("tab", name=filename).get_by_text(filename).click()

        for i in range(retry):
            self._restart_run_all()

            output_locator = self.nav.page.get_by_text(expected_output_text, exact=True)
            with contextlib.suppress(Exception):
                if output_locator.is_visible():
                    break

    def _restart_run_all(self):
        # restart run all cells
        self.nav.page.get_by_text("Kernel", exact=True).click()
        self.nav.page.get_by_role(
            "menuitem", name="Restart Kernel and Run All Cells…"
        ).get_by_text("Restart Kernel and Run All Cells…").click()

        # Restart dialog appears most, but not all of the time (e.g. set
        # No Kernel, then Restart Run All)
        restart_dialog_button = self.nav.page.get_by_role(
            "button", name="Restart", exact=True
        )
        if restart_dialog_button.is_visible():
            restart_dialog_button.click()


if __name__ == "__main__":
    dotenv.load_dotenv()
    nav = Navigator(
        nebari_url="https://nebari.quansight.dev/",
        username=os.environ["KEYCLOAK_USERNAME"],
        password=os.environ["KEYCLOAK_PASSWORD"],
        auth="password",
        instance_name="small-instance",
        headless=False,
        slow_mo=100,
    )
    nav.login()
    nav.start_server()
    nav.reset_workspace()
    test_app = RunNotebook(navigator=nav)
    notebook_filepath_in_repo = "test_data/test_notebook_output.ipynb"
    notebook_filepath_on_nebari = "test_notebook_output.ipynb"
    with open(notebook_filepath_in_repo, "r") as notebook:
        test_app.nav.write_file(
            filepath=notebook_filepath_on_nebari, content=notebook.read()
        )
    test_app.run_notebook(
        path="nebari/tests_e2e/playwright/test_data/test_notebook_output.ipynb",
        expected_output_text="success: 6",
        conda_env="conda-env-default-py",
    )
    nav.teardown()



from run_notebook import RunNotebook


def test_notebook(navigator, test_data_root):
    test_app = RunNotebook(navigator=navigator)
    notebook_name = "test_notebook_output.ipynb"
    with open(test_data_root / notebook_name, "r") as notebook:
        test_app.nav.write_file(filepath=notebook_name, content=notebook.read())
    test_app.run_notebook(
        path=notebook_name,
        expected_output_text="success: 6",
        conda_env="conda-env-default-py",
        runtime=60000,
    )



CURRENT_RELEASE = "2023.7.1"

TERRAFORM_VERSION = "1.0.5"

# 04-kubernetes-ingress
DEFAULT_TRAEFIK_IMAGE_TAG = "2.9.1"

HIGHEST_SUPPORTED_K8S_VERSION = "1.24.13"
DEFAULT_GKE_RELEASE_CHANNEL = "UNSPECIFIED"

DEFAULT_NEBARI_DASK_VERSION = CURRENT_RELEASE
DEFAULT_NEBARI_IMAGE_TAG = CURRENT_RELEASE
DEFAULT_NEBARI_WORKFLOW_CONTROLLER_IMAGE_TAG = CURRENT_RELEASE

DEFAULT_CONDA_STORE_IMAGE_TAG = "v0.4.14"

LATEST_SUPPORTED_PYTHON_VERSION = "3.10"



import functools
import hashlib
import json
import os
import shutil
import sys
from pathlib import Path
from typing import Dict, List

import yaml
from rich import print
from rich.table import Table
from ruamel.yaml import YAML

import _nebari
from _nebari.deprecate import DEPRECATED_FILE_PATHS
from _nebari.provider.cicd.github import gen_nebari_linter, gen_nebari_ops
from _nebari.provider.cicd.gitlab import gen_gitlab_ci
from _nebari.stages import tf_objects
from _nebari.utils import is_relative_to


def render_template(output_directory, config_filename, dry_run=False):
    # get directory for nebari templates
    template_directory = Path(_nebari.__file__).parent / "template"

    # would be nice to remove assumption that input directory
    # is in local filesystem and a directory
    if not template_directory.is_dir():
        raise ValueError(f"template directory={template_directory} is not a directory")

    if output_directory == Path.home():
        print("ERROR: Deploying Nebari in home directory is not advised!")
        sys.exit(1)

    # mkdir all the way down to repo dir so we can copy .gitignore
    # into it in remove_existing_renders
    output_directory.mkdir(exist_ok=True, parents=True)

    if not config_filename.is_file():
        raise ValueError(
            f"cookiecutter configuration={config_filename} is not filename"
        )

    with open(config_filename) as f:
        yaml = YAML(typ="safe", pure=True)
        config = yaml.load(f)

    # For any config values that start with
    # NEBARI_SECRET_, set the values using the
    # corresponding env var.
    set_env_vars_in_config(config)

    config["repo_directory"] = output_directory.name
    config["nebari_config_yaml_path"] = str(config_filename.absolute())

    contents = render_contents(config)

    directories = [
        f"stages/02-infrastructure/{config['provider']}",
        "stages/03-kubernetes-initialize",
        "stages/04-kubernetes-ingress",
        "stages/05-kubernetes-keycloak",
        "stages/06-kubernetes-keycloak-configuration",
        "stages/07-kubernetes-services",
        "stages/08-nebari-tf-extensions",
    ]
    if (
        config["provider"] not in {"existing", "local"}
        and config["terraform_state"]["type"] == "remote"
    ):
        directories.append(f"stages/01-terraform-state/{config['provider']}")

    source_dirs = [template_directory / Path(directory) for directory in directories]
    output_dirs = [output_directory / Path(directory) for directory in directories]
    new, untracked, updated, deleted = inspect_files(
        source_dirs,
        output_dirs,
        source_base_dir=template_directory,
        output_base_dir=output_directory,
        ignore_filenames=[
            "terraform.tfstate",
            ".terraform.lock.hcl",
            "terraform.tfstate.backup",
        ],
        ignore_directories=[
            ".terraform",
            "__pycache__",
        ],
        deleted_paths=DEPRECATED_FILE_PATHS,
        contents=contents,
    )

    if new:
        table = Table("The following files will be created:", style="deep_sky_blue1")
        for filename in sorted(new):
            table.add_row(str(filename), style="green")
        print(table)
    if updated:
        table = Table("The following files will be updated:", style="deep_sky_blue1")
        for filename in sorted(updated):
            table.add_row(str(filename), style="green")
        print(table)
    if deleted:
        table = Table("The following files will be deleted:", style="deep_sky_blue1")
        for filename in sorted(deleted):
            table.add_row(str(filename), style="green")
        print(table)
    if untracked:
        table = Table(
            "The following files are untracked (only exist in output directory):",
            style="deep_sky_blue1",
        )
        for filename in sorted(updated):
            table.add_row(str(filename), style="green")
        print(table)

    if dry_run:
        print("dry-run enabled no files will be created, updated, or deleted")
    else:
        for filename in new | updated:
            input_filename = template_directory / filename
            output_filename = output_directory / filename
            output_filename.parent.mkdir(parents=True, exist_ok=True)

            if input_filename.exists():
                shutil.copy(input_filename, output_filename)
            else:
                with open(output_filename, "w") as f:
                    f.write(contents[filename])

        for path in deleted:
            abs_path = (output_directory / path).resolve()

            if not is_relative_to(abs_path, output_directory):
                raise Exception(
                    f"[ERROR] SHOULD NOT HAPPEN filename was about to be deleted but path={abs_path} is outside of output_directory"
                )

            if abs_path.is_file():
                abs_path.unlink()
            elif abs_path.is_dir():
                shutil.rmtree(abs_path)


def render_contents(config: Dict):
    """Dynamically generated contents from _nebari configuration."""
    contents = {
        **tf_objects.stage_01_terraform_state(config),
        **tf_objects.stage_02_infrastructure(config),
        **tf_objects.stage_03_kubernetes_initialize(config),
        **tf_objects.stage_04_kubernetes_ingress(config),
        **tf_objects.stage_05_kubernetes_keycloak(config),
        **tf_objects.stage_06_kubernetes_keycloak_configuration(config),
        **tf_objects.stage_07_kubernetes_services(config),
        **tf_objects.stage_08_nebari_tf_extensions(config),
    }

    if config.get("ci_cd"):
        for fn, workflow in gen_cicd(config).items():
            workflow_json = workflow.json(
                indent=2,
                by_alias=True,
                exclude_unset=True,
                exclude_defaults=True,
            )
            workflow_yaml = yaml.dump(
                json.loads(workflow_json), sort_keys=False, indent=2
            )
            contents.update({fn: workflow_yaml})

    contents.update(gen_gitignore())

    return contents


def gen_gitignore():
    """
    Generate `.gitignore` file.
    Add files as needed.
    """
    from inspect import cleandoc

    files_to_ignore = """
        # ignore terraform state
        .terraform
        terraform.tfstate
        terraform.tfstate.backup
        .terraform.tfstate.lock.info

        # python
        __pycache__
    """
    return {Path(".gitignore"): cleandoc(files_to_ignore)}


def gen_cicd(config):
    """
    Use cicd schema to generate workflow files based on the
    `ci_cd` key in the `config`.

    For more detail on schema:
    GiHub-Actions - nebari/providers/cicd/github.py
    GitLab-CI - nebari/providers/cicd/gitlab.py
    """
    cicd_files = {}
    cicd_provider = config["ci_cd"]["type"]

    if cicd_provider == "github-actions":
        gha_dir = Path(".github") / "workflows"
        cicd_files[gha_dir / "nebari-ops.yaml"] = gen_nebari_ops(config)
        cicd_files[gha_dir / "nebari-linter.yaml"] = gen_nebari_linter(config)

    elif cicd_provider == "gitlab-ci":
        cicd_files[Path(".gitlab-ci.yml")] = gen_gitlab_ci(config)

    else:
        raise ValueError(
            f"The ci_cd provider, {cicd_provider}, is not supported. Supported providers include: `github-actions`, `gitlab-ci`."
        )

    return cicd_files


def inspect_files(
    source_dirs: Path,
    output_dirs: Path,
    source_base_dir: Path,
    output_base_dir: Path,
    ignore_filenames: List[str] = None,
    ignore_directories: List[str] = None,
    deleted_paths: List[Path] = None,
    contents: Dict[str, str] = None,
):
    """Return created, updated and untracked files by computing a checksum over the provided directory.

    Args:
        source_dirs (Path): The source dir used as base for comparison
        output_dirs (Path): The destination dir which will be matched with
        source_base_dir (Path): Relative base path to source directory
        output_base_dir (Path): Relative base path to output directory
        ignore_filenames (list[str]): Filenames to ignore while comparing for changes
        ignore_directories (list[str]): Directories to ignore while comparing for changes
        deleted_paths (list[Path]): Paths that if exist in output directory should be deleted
        contents (dict): filename to content mapping for dynamically generated files
    """
    ignore_filenames = ignore_filenames or []
    ignore_directories = ignore_directories or []
    contents = contents or {}

    source_files = {}
    output_files = {}

    def list_files(
        directory: Path, ignore_filenames: List[str], ignore_directories: List[str]
    ):
        for path in directory.rglob("*"):
            if not path.is_file():
                continue

            if path.name in ignore_filenames:
                continue

            if any(
                d in ignore_directories for d in path.relative_to(directory).parts[:-1]
            ):
                continue

            yield path

    for filename, content in contents.items():
        source_files[filename] = hashlib.sha256(content.encode("utf8")).hexdigest()
        output_filename = output_base_dir / filename
        if output_filename.is_file():
            output_files[filename] = hash_file(filename)

    deleted_files = set()
    for path in deleted_paths:
        absolute_path = output_base_dir / path
        if absolute_path.exists():
            deleted_files.add(path)

    for source_dir, output_dir in zip(source_dirs, output_dirs):
        for filename in list_files(source_dir, ignore_filenames, ignore_directories):
            relative_path = filename.relative_to(source_base_dir)
            source_files[relative_path] = hash_file(filename)

        for filename in list_files(output_dir, ignore_filenames, ignore_directories):
            relative_path = filename.relative_to(output_base_dir)
            output_files[relative_path] = hash_file(filename)

    new_files = source_files.keys() - output_files.keys()
    untracted_files = output_files.keys() - source_files.keys()

    updated_files = set()
    for prevalent_file in source_files.keys() & output_files.keys():
        if source_files[prevalent_file] != output_files[prevalent_file]:
            updated_files.add(prevalent_file)

    return new_files, untracted_files, updated_files, deleted_paths


def hash_file(file_path: str):
    """Get the hex digest of the given file.

    Args:
        file_path (str): path to file
    """
    with open(file_path, "rb") as f:
        return hashlib.sha256(f.read()).hexdigest()


def set_env_vars_in_config(config):
    """

    For values in the config starting with 'NEBARI_SECRET_XXX' the environment
    variables are searched for the pattern XXX and the config value is
    modified. This enables setting secret values that should not be directly
    stored in the config file.

    NOTE: variables are most likely written to a file somewhere upon render. In
    order to further reduce risk of exposure of any of these variables you might
    consider preventing storage of the terraform render output.
    """
    private_entries = get_secret_config_entries(config)
    for idx in private_entries:
        set_nebari_secret(config, idx)


def get_secret_config_entries(config, config_idx=None, private_entries=None):
    output = private_entries or []
    if config_idx is None:
        sub_dict = config
        config_idx = []
    else:
        sub_dict = get_sub_config(config, config_idx)

    for key, value in sub_dict.items():
        if type(value) is dict:
            sub_dict_outputs = get_secret_config_entries(
                config, [*config_idx, key], private_entries
            )
            output = [*output, *sub_dict_outputs]
        else:
            if "NEBARI_SECRET_" in str(value):
                output = [*output, [*config_idx, key]]
    return output


def get_sub_config(conf, conf_idx):
    sub_config = functools.reduce(dict.__getitem__, conf_idx, conf)
    return sub_config


def set_sub_config(conf, conf_idx, value):
    get_sub_config(conf, conf_idx[:-1])[conf_idx[-1]] = value


def set_nebari_secret(config, idx):
    placeholder = get_sub_config(config, idx)
    secret_var = get_nebari_secret(placeholder)
    set_sub_config(config, idx, secret_var)


def get_nebari_secret(secret_var):
    env_var = secret_var.lstrip("NEBARI_SECRET_")
    val = os.environ.get(env_var)
    if not val:
        raise EnvironmentError(
            f"Since '{secret_var}' was found in the"
            " Nebari config, the environment variable"
            f" '{env_var}' must be set."
        )
    return val



import subprocess

import pytest

from _nebari.schema import InitInputs
from _nebari.utils import load_yaml

PROJECT_NAME = "clitest"
DOMAIN_NAME = "clitest.dev"


@pytest.mark.parametrize(
    "namespace, auth_provider, ci_provider, ssl_cert_email",
    (
        [None, None, None, None],
        ["prod", "github", "github-actions", "it@acme.org"],
    ),
)
def test_nebari_init(tmp_path, namespace, auth_provider, ci_provider, ssl_cert_email):
    """Test `nebari init` CLI command."""
    command = [
        "nebari",
        "init",
        "local",
        f"--project={PROJECT_NAME}",
        f"--domain={DOMAIN_NAME}",
        "--disable-prompt",
    ]

    default_values = InitInputs()

    if namespace:
        command.append(f"--namespace={namespace}")
    else:
        namespace = default_values.namespace
    if auth_provider:
        command.append(f"--auth-provider={auth_provider}")
    else:
        auth_provider = default_values.auth_provider
    if ci_provider:
        command.append(f"--ci-provider={ci_provider}")
    else:
        ci_provider = default_values.ci_provider
    if ssl_cert_email:
        command.append(f"--ssl-cert-email={ssl_cert_email}")
    else:
        ssl_cert_email = default_values.ssl_cert_email

    subprocess.run(command, cwd=tmp_path, check=True)

    config = load_yaml(tmp_path / "nebari-config.yaml")

    assert config.get("namespace") == namespace
    assert (
        config.get("security", {}).get("authentication", {}).get("type").lower()
        == auth_provider
    )
    ci_cd = config.get("ci_cd", None)
    if ci_cd:
        assert ci_cd.get("type", {}) == ci_provider
    else:
        assert ci_cd == ci_provider
    acme_email = config.get("certificate", None)
    if acme_email:
        assert acme_email.get("acme_email") == ssl_cert_email
    else:
        assert acme_email == ssl_cert_email


def test_python_invocation():
    def run(command):
        return subprocess.run(
            command, check=True, capture_output=True, text=True
        ).stdout.strip()

    command = ["nebari", "--version"]

    actual = run(["python", "-m", *command])
    expected = run(command)

    assert actual == expected



import logging
import os
from pathlib import Path

import dotenv
import pytest
from navigator import Navigator

logger = logging.getLogger()


@pytest.fixture(scope="session")
def _navigator_session(browser_name, pytestconfig):
    """Set up a navigator instance, login with username/password, start
    a server. Teardown when session is complete.
    Do not use this for individual tests, use `navigator` fixture
    for tests."""
    dotenv.load_dotenv()
    # try/except added here in attempt to reach teardown after error in
    # order to close the browser context which will save the video so I debug
    # the error.
    try:
        nav = Navigator(
            nebari_url=os.environ["NEBARI_FULL_URL"],
            username=os.environ["KEYCLOAK_USERNAME"],
            password=os.environ["KEYCLOAK_PASSWORD"],
            headless=not pytestconfig.getoption("--headed"),
            slow_mo=pytestconfig.getoption("--slowmo"),
            browser=browser_name,
            auth="password",
            instance_name="small-instance",  # small-instance included by default
            video_dir="videos/",
        )
    except Exception as e:
        logger.debug(e)
        raise

    try:
        nav.login_password()
        nav.start_server()
        yield nav
    except Exception as e:
        logger.debug(e)
        raise
    finally:
        nav.teardown()


@pytest.fixture(scope="function")
def navigator(_navigator_session):
    """High level navigator instance with a reset workspace."""
    _navigator_session.reset_workspace()
    yield _navigator_session


@pytest.fixture(scope="session")
def test_data_root():
    here = Path(__file__).parent
    return here / "test_data"



from pathlib import Path
from typing import Optional
from zipfile import ZipFile

import typer
from click import Context
from kubernetes import client
from kubernetes import config as kube_config
from rich import print
from ruamel import yaml
from typer.core import TyperGroup

from _nebari.cli.dev import app_dev
from _nebari.cli.init import (
    check_auth_provider_creds,
    check_cloud_provider_creds,
    check_project_name,
    check_ssl_cert_email,
    enum_to_list,
    guided_init_wizard,
    handle_init,
)
from _nebari.cli.keycloak import app_keycloak
from _nebari.deploy import deploy_configuration
from _nebari.destroy import destroy_configuration
from _nebari.render import render_template
from _nebari.schema import (
    AuthenticationEnum,
    CiEnum,
    GitRepoEnum,
    InitInputs,
    ProviderEnum,
    TerraformStateEnum,
    verify,
)
from _nebari.upgrade import do_upgrade
from _nebari.utils import load_yaml
from _nebari.version import __version__

SECOND_COMMAND_GROUP_NAME = "Additional Commands"
GUIDED_INIT_MSG = (
    "[bold green]START HERE[/bold green] - this will guide you step-by-step "
    "to generate your [purple]nebari-config.yaml[/purple]. "
    "It is an [i]alternative[/i] to passing the options listed below."
)
KEYCLOAK_COMMAND_MSG = (
    "Interact with the Nebari Keycloak identity and access management tool."
)
DEV_COMMAND_MSG = "Development tools and advanced features."


def path_callback(value: str) -> Path:
    return Path(value).expanduser().resolve()


def config_path_callback(value: str) -> Path:
    value = path_callback(value)
    if not value.is_file():
        raise ValueError(f"Passed configuration path {value} does not exist!")
    return value


CONFIG_PATH_OPTION: Path = typer.Option(
    ...,
    "--config",
    "-c",
    help="nebari configuration yaml file path, please pass in as -c/--config flag",
    callback=config_path_callback,
)

OUTPUT_PATH_OPTION: Path = typer.Option(
    Path.cwd(),
    "-o",
    "--output",
    help="output directory",
    callback=path_callback,
)


class OrderCommands(TyperGroup):
    def list_commands(self, ctx: Context):
        """Return list of commands in the order appear."""
        return list(self.commands)


app = typer.Typer(
    cls=OrderCommands,
    help="Nebari CLI 🪴",
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
)
app.add_typer(
    app_keycloak,
    name="keycloak",
    help=KEYCLOAK_COMMAND_MSG,
    rich_help_panel=SECOND_COMMAND_GROUP_NAME,
)
app.add_typer(
    app_dev,
    name="dev",
    help=DEV_COMMAND_MSG,
    rich_help_panel=SECOND_COMMAND_GROUP_NAME,
)


@app.callback(invoke_without_command=True)
def version(
    version: Optional[bool] = typer.Option(
        None,
        "-V",
        "--version",
        help="Nebari version number",
        is_eager=True,
    ),
):
    if version:
        print(__version__)
        raise typer.Exit()


@app.command()
def init(
    cloud_provider: str = typer.Argument(
        "local",
        help=f"options: {enum_to_list(ProviderEnum)}",
        callback=check_cloud_provider_creds,
        is_eager=True,
    ),
    # Although this unused below, the functionality is contained in the callback. Thus,
    # this attribute cannot be removed.
    guided_init: bool = typer.Option(
        False,
        help=GUIDED_INIT_MSG,
        callback=guided_init_wizard,
        is_eager=True,
    ),
    project_name: str = typer.Option(
        ...,
        "--project-name",
        "--project",
        "-p",
        callback=check_project_name,
    ),
    domain_name: str = typer.Option(
        ...,
        "--domain-name",
        "--domain",
        "-d",
    ),
    namespace: str = typer.Option(
        "dev",
    ),
    auth_provider: str = typer.Option(
        "password",
        help=f"options: {enum_to_list(AuthenticationEnum)}",
        callback=check_auth_provider_creds,
    ),
    auth_auto_provision: bool = typer.Option(
        False,
    ),
    repository: str = typer.Option(
        None,
        help=f"options: {enum_to_list(GitRepoEnum)}",
    ),
    repository_auto_provision: bool = typer.Option(
        False,
    ),
    ci_provider: str = typer.Option(
        None,
        help=f"options: {enum_to_list(CiEnum)}",
    ),
    terraform_state: str = typer.Option(
        "remote", help=f"options: {enum_to_list(TerraformStateEnum)}"
    ),
    kubernetes_version: str = typer.Option(
        "latest",
    ),
    ssl_cert_email: str = typer.Option(
        None,
        callback=check_ssl_cert_email,
    ),
    disable_prompt: bool = typer.Option(
        False,
        is_eager=True,
    ),
):
    """
    Create and initialize your [purple]nebari-config.yaml[/purple] file.

    This command will create and initialize your [purple]nebari-config.yaml[/purple] :sparkles:

    This file contains all your Nebari cluster configuration details and,
    is used as input to later commands such as [green]nebari render[/green], [green]nebari deploy[/green], etc.

    If you're new to Nebari, we recommend you use the Guided Init wizard.
    To get started simply run:

            [green]nebari init --guided-init[/green]

    """
    inputs = InitInputs()

    inputs.cloud_provider = cloud_provider
    inputs.project_name = project_name
    inputs.domain_name = domain_name
    inputs.namespace = namespace
    inputs.auth_provider = auth_provider
    inputs.auth_auto_provision = auth_auto_provision
    inputs.repository = repository
    inputs.repository_auto_provision = repository_auto_provision
    inputs.ci_provider = ci_provider
    inputs.terraform_state = terraform_state
    inputs.kubernetes_version = kubernetes_version
    inputs.ssl_cert_email = ssl_cert_email
    inputs.disable_prompt = disable_prompt

    handle_init(inputs)


@app.command(rich_help_panel=SECOND_COMMAND_GROUP_NAME)
def validate(
    config_path=CONFIG_PATH_OPTION,
    enable_commenting: bool = typer.Option(
        False, "--enable-commenting", help="Toggle PR commenting on GitHub Actions"
    ),
):
    """
    Validate the values in the [purple]nebari-config.yaml[/purple] file are acceptable.
    """
    config = load_yaml(config_path)

    if enable_commenting:
        # for PR's only
        # comment_on_pr(config)
        pass
    else:
        verify(config)
        print("[bold purple]Successfully validated configuration.[/bold purple]")


@app.command(rich_help_panel=SECOND_COMMAND_GROUP_NAME)
def render(
    output_path=OUTPUT_PATH_OPTION,
    config_path=CONFIG_PATH_OPTION,
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        help="simulate rendering files without actually writing or updating any files",
    ),
):
    """
    Dynamically render the Terraform scripts and other files from your [purple]nebari-config.yaml[/purple] file.
    """
    config = load_yaml(config_path)

    verify(config)

    render_template(output_path, config_path, dry_run=dry_run)


@app.command()
def deploy(
    config_path=CONFIG_PATH_OPTION,
    output_path=OUTPUT_PATH_OPTION,
    dns_provider: str = typer.Option(
        False,
        "--dns-provider",
        help="dns provider to use for registering domain name mapping",
    ),
    dns_auto_provision: bool = typer.Option(
        False,
        "--dns-auto-provision",
        help="Attempt to automatically provision DNS, currently only available for `cloudflare`",
    ),
    disable_prompt: bool = typer.Option(
        False,
        "--disable-prompt",
        help="Disable human intervention",
    ),
    disable_render: bool = typer.Option(
        False,
        "--disable-render",
        help="Disable auto-rendering in deploy stage",
    ),
    disable_checks: bool = typer.Option(
        False,
        "--disable-checks",
        help="Disable the checks performed after each stage",
    ),
    skip_remote_state_provision: bool = typer.Option(
        False,
        "--skip-remote-state-provision",
        help="Skip terraform state deployment which is often required in CI once the terraform remote state bootstrapping phase is complete",
    ),
):
    """
    Deploy the Nebari cluster from your [purple]nebari-config.yaml[/purple] file.
    """
    config = load_yaml(config_path)

    verify(config)

    if not disable_render:
        render_template(output_path, config_path)

    deploy_configuration(
        config,
        dns_provider=dns_provider,
        dns_auto_provision=dns_auto_provision,
        disable_prompt=disable_prompt,
        disable_checks=disable_checks,
        skip_remote_state_provision=skip_remote_state_provision,
    )


@app.command()
def destroy(
    config_path=CONFIG_PATH_OPTION,
    output_path=OUTPUT_PATH_OPTION,
    disable_render: bool = typer.Option(
        False,
        "--disable-render",
        help="Disable auto-rendering before destroy",
    ),
    disable_prompt: bool = typer.Option(
        False,
        "--disable-prompt",
        help="Destroy entire Nebari cluster without confirmation request. Suggested for CI use.",
    ),
):
    """
    Destroy the Nebari cluster from your [purple]nebari-config.yaml[/purple] file.
    """

    def _run_destroy(config_path=config_path, disable_render=disable_render):
        config = load_yaml(config_path)

        verify(config)

        if not disable_render:
            render_template(output_path, config_path)

        destroy_configuration(config)

    if disable_prompt:
        _run_destroy()
    elif typer.confirm("Are you sure you want to destroy your Nebari cluster?"):
        _run_destroy()
    else:
        raise typer.Abort()


@app.command(rich_help_panel=SECOND_COMMAND_GROUP_NAME)
def upgrade(
    config_path=CONFIG_PATH_OPTION,
    attempt_fixes: bool = typer.Option(
        False,
        "--attempt-fixes",
        help="Attempt to fix the config for any incompatibilities between your old and new Nebari versions.",
    ),
):
    """
    Upgrade your [purple]nebari-config.yaml[/purple].

    Upgrade your [purple]nebari-config.yaml[/purple] after an nebari upgrade. If necessary, prompts users to perform manual upgrade steps required for the deploy process.

    See the project [green]RELEASE.md[/green] for details.
    """
    do_upgrade(config_path, attempt_fixes=attempt_fixes)


@app.command(rich_help_panel=SECOND_COMMAND_GROUP_NAME)
def support(
    config_path=CONFIG_PATH_OPTION,
    output_path=OUTPUT_PATH_OPTION,
):
    """
    Support tool to write all Kubernetes logs locally and compress them into a zip file.

    The Nebari team recommends k9s to manage and inspect the state of the cluster.
    However, this command occasionally helpful for debugging purposes should the logs need to be shared.
    """
    kube_config.load_kube_config()

    v1 = client.CoreV1Api()

    namespace = get_config_namespace(config_path)

    pods = v1.list_namespaced_pod(namespace=namespace)

    for pod in pods.items:
        Path(f"./log/{namespace}").mkdir(parents=True, exist_ok=True)
        path = Path(f"./log/{namespace}/{pod.metadata.name}.txt")
        with path.open(mode="wt") as file:
            try:
                file.write(
                    "%s\t%s\t%s\n"
                    % (
                        pod.status.pod_ip,
                        namespace,
                        pod.metadata.name,
                    )
                )

                # some pods are running multiple containers
                containers = [
                    _.name if len(pod.spec.containers) > 1 else None
                    for _ in pod.spec.containers
                ]

                for container in containers:
                    if container is not None:
                        file.write(f"Container: {container}\n")
                    file.write(
                        v1.read_namespaced_pod_log(
                            name=pod.metadata.name,
                            namespace=namespace,
                            container=container,
                        )
                    )

            except client.exceptions.ApiException as e:
                file.write("%s not available" % pod.metadata.name)
                raise e

    with ZipFile(output_path, "w") as zip:
        for file in list(Path(f"./log/{namespace}").glob("*.txt")):
            print(file)
            zip.write(file)


def get_config_namespace(config_path):
    with open(config_path) as f:
        config = yaml.safe_load(f.read())

    return config["namespace"]


if __name__ == "__main__":
    app()
